--- Internal sync interface module
-- @author Oskar Eisemuth
-- @copyright 2019
-- @module commonapi._syncdata

local syncdata = {}

local _utils = commonapi.utils
local _log = _utils.logPrep("commonapi2._syncdata")

local syncdebug = false


local _shareddataptr


if (commonapi._native ~= nil and commonapi._native.shareddata ~= nil) then
	_shareddataptr = commonapi._native.shareddata
end

local SyncObject = {
	key = "",
	serial = 0,
	data = false
}

--- Sync Primitive constructor
-- @tparam[opt] table o A table of settings
-- @treturn SyncObject
function SyncObject:new(o)
	local o = o or {}

	for k, v in pairs(self) do
		if (o[k] == nil) then
			o[k] = _utils.deepcopy(v)
		end
	end

	o._super = self
	return o
end


--function SyncObject:hasNewData()
--	if (_shareddataptr == nil) then
--		return false
--	end
--end

function SyncObject:getData()
	if (_shareddataptr == nil) then
		return self.data or nil
	end

	local newserial =_shareddataptr.getSerial(self.key)
	if (newserial ~= nil) then
		if (newserial ~= self.serial) then
			if (syncdebug) then
				_log("Syncing shareddata ("..self.key..") get new: ".. (newserial or 'nil ') .. " old: " .. (self.serial or 'nil ') .. " ".. commonapi._native.getLuaStatePtr() )
			end
			local data = _shareddataptr.get(self.key)

			if (self.unserialize ~= nil) then
				self.data = self.unserialize(self, data)
			else
				self.data = data
			end
		end
	end
	return self.data or nil
end

function SyncObject:getDataOrInit(initdata)
	local data = self:getData()
	if (data == nil) then
		self.data = _utils.deepcopy(initdata)
	end
	return self.data
end

function SyncObject:syncData(data)
	if (_shareddataptr == nil) then
		return
	end
	if (data == nil) then
		data = self.data
	end
	if (self.serialize ~= nil) then
		local serdata
		serdata = self.serialize(self, data)
		self.serial = _shareddataptr.set(self.key, serdata)
	else
		self.serial = _shareddataptr.set(self.key, data)
	end
	self.data = data
	if (syncdebug) then
		_log("Syncing shareddata ("..self.key..") done, new serial: ".. (self.serial or 'nil ') .. " ".. commonapi._native.getLuaStatePtr() )
	end
end

syncdata.SyncObject = SyncObject
return syncdata