local ModCommonListWindow = {}

local _log = commonapi.utils.logPrep("commonapi2.ui.ModCommonListWindow")
local _utils = commonapi.utils
local _ = _utils._t
local ui = commonapi.ui;


--commonapi._reloadModule("_modmm")
local modmm = commonapi._modmm;

ModCommonListWindow = ui.Window:newClass()
ModCommonListWindow.caption = "ModCommonListWindow"


function ModCommonListWindow:checkExtractor(uilist)
	if (self.params.missingextractor == nil) then
		_log("Checking 7zip extractor")
		self.params.missingextractor_path = modmm.getExtractorBin()
		local zipok, zipmsg  = modmm.extractorIsUsable()
		self.params.missingextractor = not zipok
		self.params.missingextractor_msg = zipmsg
	end

	--if (self.params.missingextractor == true) then
		uilist[#uilist+1] = {
			typ = "Label",
			id = "extractorMissing",
			caption = _("7zip (${path}), ${msg}") % {
				path = self.params.missingextractor_path or "",
				msg = self.params.missingextractor_msg or ""
			},
		}
	if (self.params.missingextractor == true) then
		uilist[#uilist+1] = {
			typ = "SmallButton",
			id = "extractorMissingDoDownload",
			caption = _("Download 7zip"),
			sameline = true,
			onclick = function(item, win)
				ui.msgbox("DoInstallExtractor",	_("Download 7zip?") , "Download 7zip", {
					{ caption = _("Cancel") }, { caption = _("Download and extract..."), fn = function()
						local ok, errormsg = modmm.extractorDownload()
						if (ok == false) then
							ui.msgbox('ModListWindow_downloadExtractor', errormsg, "Download 7zip")
						end
						win.params.missingextractor = nil
						win:clearCache()
					end
				},
			})
			end
		}
	end
end

function ModCommonListWindow:genModUIList()
	local uilist = {}
	self.uilist = uilist

	if (self._modmmrepos == nil) then
		self._modmmrepos = modmm.getRepoDataObj()
		self._modmmrepos:reload()
	end


	return uilist
end


function ModCommonListWindow:genRepoDownloadList(uilist, repoentries)
	for mu, repoentry in ipairs( repoentries ) do

		local caption = repoentry.name..' - ('..table.concat(repoentry.version, ".")..')  via '..repoentry.repodata.name
		if (repoentry.utc_changed ~= nil) then
			caption = caption .. '  ' .. os.date('%Y-%m-%d', repoentry.utc_changed)
		end

		uilist[#uilist+1] = {
			typ = "_Selectable",
			id = repoentry.modid..'__repo__'..mu,
			caption = caption,
			data = {
				modid = repoentry.modid,
				repoentry = repoentry,
			},
			onclick = function(item, win)
				ui.msgbox("DoModUpgrade",
					_("Install ${modname}?") % {
						modname = repoentry.name..' - ('..table.concat(repoentry.version, ".")..')'
					}
					, "Mod upgrade", {
						{ caption = _("Cancel") }, { caption = _("Install or Upgrade"), fn = function()
							local ok, errormsg = modmm.downloadAndInstallMod(item.data.repoentry)
							if (ok == false) then
								ui.msgbox('ModListWindow_downloadAndInstallMod', "Fehler beim Download/Update von:"..repoentry.modid.." \n"..errormsg, "Update")
							end
							win:clearCache()
						end
					},
				})
			end,
		}
	end
end


function ModCommonListWindow:genUpgradeList(uilist, entry)
	for mu, repoentry in ipairs( entry.repomods ) do

		local caption = repoentry.name..' - ('..table.concat(repoentry.version, ".")..')    via '..repoentry.repodata.name
		if (repoentry.utc_changed ~= nil) then
			caption = caption .. '  ' .. os.date('%Y-%m-%d', repoentry.utc_changed)
		end
		uilist[#uilist+1] = {
			typ = "_Selectable",
			id = entry.id..'__repo__'..mu,
			caption = caption,
			data = {
				modid = entry.id,
				repoentry = repoentry,
			},
			onclick = function(item, win)
				--entry.downloadfull = repodata.download_prefix_url..''..entry.download
				--entry.entryurlfull = repodata.entry_prefix_url..''..entry.entryurl
				--commonapi._native.utils.openurl(item.data.repodata.download_prefix_url..item.data.repoentry.download)
				ui.msgbox("DoModUpgrade",
					_("Install ${modname}?") % {
						modname = repoentry.name..' - ('..table.concat(repoentry.version, ".")..')'
					}
					, "Mod upgrade", {
						{ caption = _("Cancel") }, { caption = _("Upgrade"), fn = function()
							local ok, errormsg = modmm.downloadAndInstallMod(item.data.repoentry)
							if (ok == false) then
								ui.msgbox('ModListWindow_downloadAndInstallMod', "Fehler beim Download/Update von:"..entry.id.." \n"..errormsg, "Update")
							end
							win:clearCache()
						end
					},
				})
			end,
		}
	end
end


function ModCommonListWindow:genModMMDownloadLink(uilist, repoentry)
	local caption = repoentry.name..' - ('..table.concat(repoentry.version, ".")..')    via '..repoentry.repodata.name
	uilist[#uilist+1] = {
		typ = "Button",
		id = repoentry.modid..'__modmmdownload__',
		caption = caption,
		data = {
			repoentry = repoentry,
		},
		onclick = function(item, win)
			--entry.downloadfull = repodata.download_prefix_url..''..entry.download
			--entry.entryurlfull = repodata.entry_prefix_url..''..entry.entryurl
			--commonapi._native.utils.openurl(item.data.repodata.download_prefix_url..item.data.repoentry.download)
			ui.msgbox("DoModUpgrade",
				_("Install ${modname}?") % {
					modname = repoentry.name..' - ('..table.concat(repoentry.version, ".")..')'
				}
				, "Mod upgrade", {
					{ caption = _("Cancel") }, { caption = _("Upgrade or Downlad"), fn = function()
						local ok, errormsg = modmm.downloadAndInstallMod(item.data.repoentry)
						if (ok == false) then
							ui.msgbox('ModListWindow_downloadAndInstallMod', "Fehler beim Download/Update von:"..caption.." \n"..errormsg, "Update")
						end
						win:clearCache()
					end
				},
			})
		end,
	}
end

function ModCommonListWindow:genDepList(uilist, modentry)
	local requiredMods = modentry.data.info.requiredMods or {}

	for m, entry in pairs( requiredMods ) do

		local links = commonapi.mods._checkModDepends_createLinks(entry)

		local uimodid = {
			typ = "Label",
			caption = ""..(entry.modId or "No modId").." , "..(entry.steamId or "No SteamId")
		}
		uilist[#uilist+1] = uimodid
		local uiversion = {
			typ = "Label",
			caption = "\tMinorVersion: >="..(entry.minMinorVersion or 0)
		}
		uilist[#uilist+1] = uiversion

		if (entry.modId ~= nil and self._modmmrepos ~= nil) then
			local repoentries = self._modmmrepos:findEntriesByModId(entry.modId)
			if (repoentries ~= nil and #repoentries > 0) then
				ModCommonListWindow.genModMMDownloadLink(self, uilist, repoentries[1])
			end
		end

		for urltype, url in pairs( links ) do
			local urlentry = {
				typ = "Button",
				caption = urltype,
				sameline = true,
				id = ""..m..''..urltype,
				data = {
					myurl = url,
				},
				onclick = function(item, win)
					commonapi._native.utils.openurl(item.data.myurl)
				end
			}
			uilist[#uilist+1] = urlentry
		end
	end
end


function ModCommonListWindow:genErrorList(uilist, modentry)
	local mid = modentry.id..'__errors__'
	for i, error in ipairs( modentry.errors ) do
		local errstring = "?"
		if (error.type == "modmissing") then
			errstring = _("requires ${modId}") % {
				modId = (error.data.modId or error.data.steamId) or "?"
			}
		elseif (error.type == "modorder") then
			errstring = _("should be loaded after ${modname}") % {
				modname = error.mod.name
			}
			--errstring = "\t  ".. error.modname .. "should be loaded first"
		elseif (error.type == "modversion") then
			errstring = _("requires ${modname} ${version}") % {
				modname = error.mod.name,
				version = tostring(error.mod.version[1] or 0) .. "." .. error.data.minMinorVersion
			}
		elseif (error.type == "modinactive") then
			errstring = _("requires ${modname}, please activate") % {
				modname = error.mod.name,
			}
		else
			errstring = error.type
		end
		local uientry = {
			typ = "Label",
			caption = "\t"..errstring
		}
		uilist[#uilist+1] = uientry

		if (self._modmmrepos ~= nil and error.type == "modmissing" or error.type == "modversion") then
			if (error.data.modId ~= nil) then
				local repoentries = self._modmmrepos:findEntriesByModId(error.data.modId)
				if (repoentries ~= nil and #repoentries > 0) then
					ModCommonListWindow.genModMMDownloadLink(self, uilist, repoentries[1])
				end
			end

			if (error.links ~= nil) then
				for urltype, url in pairs( error.links ) do
					local urlentry = {
						typ = "Button",
						caption = urltype,
						sameline = true,
						id = mid..''..urltype,
						myurl = url,
						onclick = function(item, win)
							commonapi._native.utils.openurl(item.myurl)
						end
					}
					uilist[#uilist+1] = urlentry
				end
			end
		end
	end
end

function ModCommonListWindow:showRepositoryEditor(uilistparent)
	local repolist = self._modmmrepos:getRepoList()

	local uiTableData = {}
	local headerColumns = {
		{
			caption = "Filename",
			width = 100,
		},
		{ caption = "Url" },
		{
			caption =  _(""),
			width = 100,
		},
		{
			caption =  _("Last update"),
			width = 100,
		},
	}
	local TableHeader  = {
		typ = "TableHeader",
		items = headerColumns,
	}
	uiTableData[#uiTableData+1] = TableHeader

	uilistparent[#uilistparent+1] = {
		typ = "Table",
		id = "ModListWindowRepoListTable",
		columns = #headerColumns,
		border = false,
		uilist = uiTableData,
		size = {
			0, 100
		},
		flags = {
			Sortable = false,
			MultiSortable = false,
			ScrollFreezeTopRow = true
		}
	}


	for idx, repolistentry in ipairs(repolist) do
		local uientryFilename = {
			typ = "_Selectable",
			_fix_overlap = true,
			--id = repolistentry.filename..'__label',
			id = "itemRepoList_filename_"..idx,
			flags = {
				SpanAllColumns = true,
			},
			caption = repolistentry.filename,
			column = 0,
			rownew = true,
			onclick = function(item, win)
				win.data['curentryrepolist'] = repolistentry
			end,
			selected = function(item, win)
				return (win.data['curentryrepolist'] == repolistentry)
			end
		}
		uiTableData[#uiTableData+1] = uientryFilename

		local uientryURL = {
			typ = "Label",
			caption = repolistentry.url,
			column = 1,
		}
		uiTableData[#uiTableData+1] = uientryURL


		uiTableData[#uiTableData+1] = {
			typ = "SmallButton",
			id = "itemRepoList_Remove_"..idx,
			column = 2,
			caption = _("Remove"),
			data = {
				filename = repolistentry.filename,
			},
			onclick = function(item, win)
				self._modmmrepos:removeRepoFromList(item.data.filename)
				win:checkUpdates()
			end
		}


		local captionLastUpdate = ""
		
		if (repolistentry.lastupdate ~= nil) then
			captionLastUpdate = os.date('%Y-%m-%d %H:%M:%S', repolistentry.lastupdate)
		end

		local uientryLastUpdate = {
			typ = "Label",
			caption = captionLastUpdate,
			column = 3,
		}
		uiTableData[#uiTableData+1] = uientryLastUpdate
	end


	uiTableData[#uiTableData+1] = {
		typ = "TextInput",
		id = "addRepoFilename",
		column = 0,
		rownew = true,
		default = "name.repo"
	}

	uiTableData[#uiTableData+1] = {
		typ = "TextInput",
		id = "addRepoURL",
		column = 1,
		default = "https://",
	}

	uiTableData[#uiTableData+1] = {
		typ = "SmallButton",
		id = "addRepoList",
		column = 2,
		caption = _("Add"),
		onclick = function(item, win)
			local repourl = win.params['addRepoURL'] or "";
			if (repourl == "") then
				return
			end
			local repoFilename = win.params['addRepoFilename'] or "name.repo";
			self._modmmrepos:addRepoToList(repoFilename, repourl)
			win:checkUpdates()
		end
	}


	uilistparent[#uilistparent+1] = {
		typ = "SmallButton",
		id = "resetRepoList",
		caption = _("Reset"),
		onclick = function(item, win)
			self._modmmrepos:setRepoList()
			win:checkUpdates()
		end
	}



	uilistparent[#uilistparent+1] = {
		typ = "Separator",
	}


end

function ModCommonListWindow:checkUpdates()
	--local repositoryurl = "https://www.transportfever.net/filebase/repos/tpf2.json"
	local ok, errormsg = self._modmmrepos:updateRepoData()
	-- modmm.downloadRepository("tpfnet-tpf2.json", repositoryurl)
	if (ok == false) then
		ui.msgbox('ModCommonListWindow_checkUpdates', "Fehler beim Updaten: \n"..errormsg, "Download repository")
	end

	self:refresh()
end


return ModCommonListWindow