local ModListErrorWindow = {}

local _log = commonapi.utils.logPrep("commonapi2.ui.ModListErrorWindow")
local _utils = commonapi.utils
local _ = _utils._t
local ui = commonapi.ui;



local ModCommonListWindow = require("commonapi2.ui.ModCommonListWindow")

local errwin = ModCommonListWindow:newClass() -- ui.Window:new()

errwin.id = "ModListErrorWindow"
errwin.caption = _("Mod errors:")


-- called later by ModCommonListWindow genModMMDownloadLink
function errwin:clearCache()
	-- FIXME: we should tell TPF2 to refresh it's internal state somehow, but how?
	commonapi.mods._loadByFS()
	if (self:createUIList() ~= true) then
		self:close()
	end
end

function errwin:createUIList()
--	local uilist = {}

	local uilist  = self._super.genModUIList(self)

	local mods = commonapi.mods.getLoaded()
	for mid, entry in pairs( mods ) do
		if (entry.errors ~= nil) then
			if (#entry.errors > 0) then
				local uientry = {
					typ = "Label",
					caption = entry.name .. ":"
				}
				uilist[#uilist+1] = uientry
			end
			for i, error in ipairs( entry.errors ) do
				local errstring = "?"
				if (error.type == "modmissing") then
					errstring = _("requires ${modId}") % {
						modId = (error.data.modId or error.data.steamId) or "?"
					}
				elseif (error.type == "modorder") then
					errstring = _("should be loaded after ${modname}") % {
						modname = error.mod.name
					}
					--errstring = "\t  ".. error.modname .. "should be loaded first"
				elseif (error.type == "modversion") then
					errstring = _("requires ${modname} ${version}") % {
						modname = error.mod.name,
						version = tostring(error.mod.version[1] or 0) .. "." .. error.data.minMinorVersion
					}
				elseif (error.type == "modinactive") then
					errstring = _("requires ${modname}, please activate") % {
						modname = error.mod.name,
					}
				else
					errstring = error.type
				end
				local uientry = {
					typ = "Label",
					caption = "\t"..errstring
				}
				uilist[#uilist+1] = uientry


				if (self._modmmrepos ~= nil and error.data.modId ~= nil and (error.type == "modmissing" or error.type == "modversion")) then
					local repoentries = self._modmmrepos:findEntriesByModId(error.data.modId)
					if (repoentries ~= nil and #repoentries > 0) then
						self.genModMMDownloadLink(self, uilist, repoentries[1])
					end
				end

				if (error.links ~= nil) then
					for urltype, url in pairs( error.links ) do
						local urlentry = {
							typ = "Button",
							caption = urltype,
							sameline = true,
							id = mid..''..urltype,
							myurl = url,
							onclick = function(item, win)
								commonapi._native.utils.openurl(item.myurl)
							end
						}
						uilist[#uilist+1] = urlentry
					end
				end

			end
		end
	end


	local missingmods = commonapi.mods._getMissing()
	for mid, entry in pairs( missingmods ) do
		local repoentries = self._modmmrepos:findEntriesByModId(entry.id)
		if (repoentries ~= nil and #repoentries > 0) then
			local uimissing = {
				typ = "Label",
				caption = _("Missing ${modId}") % {
					modId = entry.id,
					modName = repoentries[1].name or "-"
				}
			}
			uilist[#uilist+1] = uimissing
			self.genModMMDownloadLink(self, uilist, repoentries[1])
		end
	end

	self.uilist = uilist
	if (#uilist > 0)  then
		return true
	end
	return false
end


local function savegamemodlistcallback(data, location)
	local mods = commonapi.mods
--[[
	print("savegamemodlistcallback")
	print(debug.traceback())
	print(commonapi._native.getLuaStatePtr())
	commonapi.dmp(data)
	commonapi.dmp(location)
--]]


	if (data == nil) then
		mods._reset()
		if (location == "CGame") then
			mods._shared_store()
		end
		errwin:close()

		local ModListWindow = ui.getWindow("ModListWindow")
		if (ModListWindow ~= nil) then
			ModListWindow:refresh()
		end
		return
	end

	local sfn = function(a, b)
		return a.state < b.state
	end
	table.sort(data, sfn)

	mods._reset()
	for i, entry in ipairs(data) do
		if (entry.isMissing == true) then
			mods._addMissingModId(entry.id.."_"..entry.version)
		else
			mods._addModByPathAndId(entry.path, entry.id.."_"..entry.version, (entry.state < 0) )
		end
	end
	if (location == "CGame") then
		mods._shared_store()
	end

	mods._checkMods()
	if (errwin:createUIList()) then
		errwin:open()
	else
		errwin:close()
	end


	local ModListWindow = ui.getWindow("ModListWindow")
	if (ModListWindow ~= nil) then
		ModListWindow:refresh()
	end
end

function ModListErrorWindow.run()
	commonapi._native.setModListCallbackFn(savegamemodlistcallback);
end

return ModListErrorWindow