local ModListWindow = {}

local _log = commonapi.utils.logPrep("commonapi2.ui.ModListWindow")
local _utils = commonapi.utils
local _ = _utils._t
local ui = commonapi.ui;


--commonapi._reloadModule("_modmm")
local modmm = commonapi._modmm;

local ModSettingsWindow = require("commonapi2.uicommon.ModSettingsWindow")
local ModCommonListWindow = require("commonapi2.ui.ModCommonListWindow")


ModListWindow = ModCommonListWindow:newClass()  -- ui.Window:new()

ModListWindow.id = "ModListWindow"
ModListWindow.height = 500
ModListWindow.width = 800


ModListWindow.caption = _("Mod settings")
ModListWindow.data = {}

function ModListWindow.create()
	local o
	o = ModListWindow:new()
	o:genModUIList()
	o:open()
	return o
end

function ModListWindow:refresh()
	self:genModUIList()
end


function ModListWindow:clearCache()
	commonapi.mods._loadByFS()
	self:genModUIList()
end


function ModListWindow:selectModForInstall()
	local window = self
	local FileDialog = require("commonapi2.ui.FileDialog")
	local fd = FileDialog.createOpen()
	fd.id = "FileDialog_ModInstall_Select"
	fd.caption = _("Install mod")

	local modinstallpath = commonapi._native.paths.getUserPath()
	if (modinstallpath ~= nil) then
		--modinstallpath = userpath .. "modlists"
		--commonapi._native.fs.mkdir(modlistpath)
	else
		modinstallpath = commonapi._info.path
	end

	fd.path = modinstallpath
	fd.fileext = {".zip", ".rar", ".7z", ".package"}
	fd.loadFile = function(self, filename)
		self:close()
		ModListWindow.installMod(window, filename)
	end
	fd:open()
end

function ModListWindow:installMod(filename)
	--commonapi.dmp(filename)
	--modmm.getFileInfo(filename)

	local ok, errormsg = modmm.installModFromArchiv(filename)
	if (ok == false) then
		ui.msgbox('ModListWindow:installMod', "Fehler beim installieren: \n"..errormsg, "Install Mod")
	end

end

function ModListWindow:genDepWindow(modid)
	local uilist = {}


	--local modentry = commonapi.mods.getById(modid)
	local modentry = commonapi.mods.getById(modid)
	if (modentry == nil) then
		modentry = commonapi.mods._getByIdAny(modid)
		if (modentry == nil) then
			return
		end
	end

	local requiredMods = modentry.data.info.requiredMods or {}

	local depwin = ui.Window:new()

	depwin.id = "ModDepWindow"
	depwin.caption = _("Mod dependencies:")

	for m, entry in pairs( requiredMods ) do

		local links = commonapi.mods._checkModDepends_createLinks(entry)

		local uimodid = {
			typ = "Label",
			caption = ""..(entry.modId or "No modId").." , "..(entry.steamId or "No SteamId")
		}
		uilist[#uilist+1] = uimodid
		local uiversion = {
			typ = "Label",
			caption = "\tMinorVersion: >="..(entry.minMinorVersion or 0)
		}
		uilist[#uilist+1] = uiversion

		for urltype, url in pairs( links ) do
			local urlentry = {
				typ = "Button",
				caption = urltype,
				sameline = true,
				id = ""..m..''..urltype,
				data = {
					myurl = url,
				},
				onclick = function(item, win)
					commonapi._native.utils.openurl(item.data.myurl)
				end
			}
			uilist[#uilist+1] = urlentry
		end
	end
	depwin.uilist = uilist

	depwin:open()
end

function ModListWindow:genModDetails(entry)
	if (self.uilist[#self.uilist].typ == "TabBar") then
		self.uilist[#self.uilist] = nil
	end
	local uiTabs = {}

	local uiTabBar = {
		typ = "TabBar",
		id = "ModDetailsTab",
		uilist = uiTabs
	}

	local uilistInfo ={}

	uilistInfo[#uilistInfo+1] = {
		typ = "Label",
		caption = entry.name,
	}
	uilistInfo[#uilistInfo+1] = {
		typ = "Label",
		caption = table.concat(entry.version, "."),
	}

	if (entry.path ~= nil) then
		uilistInfo[#uilistInfo+1] = {
			typ = "SmallButton",
			id = "ModDetailsTabInfoPath",
			caption = entry.path, 
			onclick = function(item, win)
				commonapi._native.utils.openfolder(entry.path)
			end
		}
	end


	uiTabs[#uiTabs+1] = {
		typ = "TabItem",
		id = "ModDetailsTabInfo",
		caption = "Details",
		uilist = uilistInfo,
	}

	local uilistDesc = {}
	if (entry.data ~= nil and entry.data.info ~= nil) then
		uilistDesc[#uilistDesc+1] = {
			typ = "Text",
			text = entry.data.info.description or "<No Description>",
		}
	end
	uiTabs[#uiTabs+1] = {
		typ = "TabItem",
		id = "ModDetailsTabDesc",
		caption = _("Description"),
		uilist = {
			{
				typ = "ScrollRegion",
				id = "ScrollRegion",
				size = {
					0, 0,
				},
				uilist = uilistDesc,
			}
		},
	}

	local requiredMods = entry.data.info.requiredMods or {}

	if (#requiredMods > 0) then
		local uilistDeps = {}
		uiTabs[#uiTabs+1] = {
			typ = "TabItem",
			id = "ModDetailsTabDep",
			caption = _("Dependencies"),
			uilist = uilistDeps,
		}
		ModCommonListWindow.genDepList(self, uilistDeps, entry)
	end

	if (entry.repomods ~= nil) then
		local uilistUpgrade = {}
		uiTabs[#uiTabs+1] = {
			typ = "TabItem",
			id = "ModDetailsTabUpgrade",
			caption = _("Upgrade"),
			uilist = {
				{
					typ = "ScrollRegion",
					id = "ScrollRegion",
					size = {
						0, 0,
					},
					uilist = uilistUpgrade,
				}
			},
		}
		ModCommonListWindow.genRepoDownloadList(self, uilistUpgrade, entry.repomods)
		--self:genUpgradeList(uilistUpgrade, entry)
	end

	if (entry.errors ~= nil and #entry.errors > 0) then
		local uilistErrors = {}
		uiTabs[#uiTabs+1] = {
			typ = "TabItem",
			id = "ModDetailsTabErrors",
			caption = _("Errors"),
			uilist = {
				{
					typ = "ScrollRegion",
					id = "ScrollRegion",
					size = {
						0, 0,
					},
					uilist = uilistErrors,
				}
			},
		}
		ModCommonListWindow.genErrorList(self, uilistErrors, entry)
	end


	self.uilist[#self.uilist+1] =  uiTabBar
end



function ModListWindow:genModUIList_CreateRow(uilist, entry)
	if (entry.data ~= nil and entry.data.info ~= nil) then
		if (entry.data.info.visible == false) then
			return
		end
	end

	local hasUpgrade = (entry.upgradeversion ~= nil)


	local uientryIdx = {
		typ = "Label",
		caption = entry._idx,
		rownew = true,
		column = 0
	}
	uilist[#uilist+1] = uientryIdx

	local uientry = {
		--typ = "Label",
		typ = "_Selectable",
		_fix_overlap = true,
		id = entry.id..'__label',
		flags = {
			SpanAllColumns = true,
		},
		caption = entry.name,
		column = 1,
		--columnwidth = 200.0,
	--	rownew = true,
		onclick = function(item, win)
			win.data['curentry'] = entry
			win:genModDetails(entry)
		end,
		selected = function(item, win)
			return (win.data['curentry'] == entry)
		end
	}

	if (hasUpgrade) then
		uientry.color = { 1.0, 1.0, 0, 1.0 }
	end

	if (entry.errors ~= nil and #entry.errors ~= 0) then
		uientry.color = { 1.0, 0.0, 0.0, 1.0 }
	end

	uilist[#uilist+1] = uientry

	if (hasUpgrade) then
		local uientryPopup = {
			typ = "SmallButton",
			id = entry.id..'_dopopup',
			caption = _("Upgrade"),
			sameline = true, 
			onclick = function(item, win)
				win.params[entry.id..'_repo_version'] = true
			end
		}
		uilist[#uilist+1] = uientryPopup
	end


	local uientryId = {
		typ = "Label",
		caption = entry.id,
		column = 2
	}
	uilist[#uilist+1] = uientryId

	local uientryVersion = {
		typ = "Label",
		caption = table.concat(entry.version, "."),
		column = 3,
		--columnoffset = 250.0,
	}
--	if (hasUpgrade) then
--		uientryVersion.caption = uientryVersion.caption .. ' - ' .. table.concat(entry.upgradeversion, ".")
--	end
	uilist[#uilist+1] = uientryVersion

	local uientryUpgradeVersion = {
		typ = "Label",
		caption = table.concat(entry.upgradeversion or {}, "."),
		column = 4,
	}
	uilist[#uilist+1] = uientryUpgradeVersion


	local popupList = {
		{
			typ = "Label",
			caption = "Click for Download"
		},
		{
			typ = "Separator",
		}
	}

	if (entry.repomods ~= nil) then
		ModCommonListWindow.genRepoDownloadList(self, popupList, entry.repomods)
		local popupRepo = {
			typ = "_Popup",
			id = entry.id..'_repo_version',
			uilist = popupList
		}
		uilist[#uilist+1] = popupRepo
	end

	local uientryButton = {
		typ = "SmallButton",
		id = entry.id..'_settings',
		caption = _("Settings"),
		column = 5,
		--columnoffset = 300.0,
		--columnoffset = self.width-60.0-100.0,
		--columnwidth = 100,
		data = {
			modid = entry.id,
		},
		onclick = function(item, win)
			--local o = ModSettingsWindow.create(item.data.modid)
			local o = ModSettingsWindow.createByModEntry(entry)
			o:open()
		end
	}

	if (commonapi.settings._getDescriptionFromEntry(entry) ~= nil) then
		uilist[#uilist+1] = uientryButton
	end


	if (entry._updatetime ~= nil) then
		local uientryLastUpdate = {
			typ = "Label",
			caption = os.date('%x', entry._updatetime),
			column = 6,
		}
		uilist[#uilist+1] = uientryLastUpdate
	end

	
--[[	
	if (entry.data.info ~= nil and entry.data.info.requiredMods ~= nil) then
		local uidepButton = {
			typ = "SmallButton",
			id = entry.id..'_dep',
			caption = _("Dep"),
			column = 3,
			--columnoffset = self.width-60.0,
			--sameline = true,
			data = {
				modid = entry.id,
			},
			onclick = function(item, win)
				win:genDepWindow(item.data.modid)
			end
		}
		uilist[#uilist+1] = uidepButton
	end
--]]
end

function ModListWindow:genModUIList()
--	local uilist = {}
	local uilist  = self._super.genModUIList(self)
	self:checkExtractor(uilist)
	
---	self._super.genModUIList(self)

	local mods
	if (self.params["showallmods"] == true) then
		if (commonapi.mods._isEmpty()) then
			commonapi.mods._loadByFS()
		end
		mods = commonapi.mods._getLoadedNotLoaded()
	else
		mods = commonapi.mods.getLoaded()
	end



	self._modmmrepos:applyRepositoriesToMods(mods)
	self.data.repotimestamp = self._modmmrepos:getLastRepoDataUpdate()
	--self.data.repotimestamp = self._modmmrepos:getUpdateStatus()
--[[
	local timestamp = modmm.applyRepositoryFilename("tpfnet-tpf2.json", mods)
	if (timestamp ~= nil) then
		self.data.repotimestamp = timestamp
	else
		self.data.repotimestamp = nil
	end
--]]

	uilist[#uilist+1] = {
		typ = "Checkbox",
		id = "showallmods",
		caption = _("Show all mods"),
		onclick = function(item, win)
			win:refresh()
		end
	}

	uilist[#uilist+1] = {
		typ = "SmallButton",
		id = "installMod",
		caption = _("Install mod"),
		sameline = true,
		onclick = function(item, win)
			win:selectModForInstall()
		end
	}

	uilist[#uilist+1] = {
		typ = "SmallButton",
		id = "checkUpdates",
		caption = _("Download update information"),
		sameline = true,
		onclick = function(item, win)
			win:checkUpdates()
		end
	}

	if (self.data.repotimestamp ~= nil) then
		uilist[#uilist+1] = {
			typ = "Label",
			caption = os.date('%Y-%m-%d %H:%M:%S', self.data.repotimestamp),
			sameline = true,
		}
	end

	uilist[#uilist+1] = {
		typ = "Checkbox",
		id = "modifyRepositoryList",
		caption = _("Repositories"),
		sameline = true,
		onclick = function(item, win)
			win:refresh()
		end
	}

	uilist[#uilist+1] = {
		typ = "SmallButton",
		id = "reloadAll",
		caption = _("Clear Cache"),
		sameline = true,
		onclick = function(item, win)
			if (commonapi._native and commonapi._native.refreshModRep ~= nil) then
				commonapi._native.refreshModRep()
			end
			win:clearCache()
		end
	}

	if (self.params["modifyRepositoryList"] == true) then
		self.showRepositoryEditor(self, uilist)
	end


--	self:genModUIList_OldTable(uilist, mods)
	uilist[#uilist+1] = {
		typ = "TextInput",
		id = "modfilter",
		changeonenter = true,
		onchange = function(item, win)
			win:refresh()
		end
	}
	self:genModUIList_NewTable(uilist, mods)
	uilist[#uilist+1] = {
		typ = "Separator"
	}
	self.uilist = uilist
end

function ModListWindow:genModUIList_NewTable(uilist, mods)
	local uiTableData = {}
	local headerColumns = {
		{
			caption = "Idx",
			width = 30,
		},
		{ caption = "Name" },
		{ caption = "Modid" },
		{
			caption = "Version",
			width = 50,
		},
		{
			caption = "U-Version",
			width = 50,
		},
		{
			caption =  _("Settings"),
			width = 100,
		},
		{
			caption =  _("Last update"),
			width = 70,
		},
	}
	local TableHeader  = {
		typ = "TableHeader",
		items = headerColumns,
		onchange = function(item, win)
			win.cursorting = item.cursorting
			win:refresh()
		end
	}
	uiTableData[#uiTableData+1] = TableHeader

	uilist[#uilist+1] = {
		typ = "Table",
		id = "ModListWindowTable",
		columns = #headerColumns,
		border = false,
		uilist = uiTableData,
		size = {
			0, -100
		},
		flags = {
			Sortable = true,
			MultiSortable = true,
			ScrollFreezeTopRow = true
		}
	}

--[[
	for m, entry in pairs( mods ) do
		self:genModUIList_CreateRow(uiTableData, entry)
	end

--]]

	local i = 1
	local filter = string.lower(self.params.modfilter or "")
	local sorted_keys = {}

	for k, entry in pairs( mods ) do
		local name_lower = string.lower(entry['name'])
		local modid_lower = string.lower(entry['id'])
		entry['name_lower'] = name_lower
		entry['modid_lower'] = modid_lower

	
		if (commonapi.settings._getDescriptionFromEntry(entry) ~= nil) then
			entry.hasSetttings = true
		else
			entry.hasSetttings = false
		end

		if (filter ~= '') then
			if ( 	(string.find( entry['name_lower'], filter) ~= nil) or 
					(string.find( entry['modid_lower'], filter) ~= nil) ) then
				sorted_keys[i] = k
				i = i + 1
			end
		else
			sorted_keys[i] = k
			i = i + 1
		end
	end





	if (self.cursorting ~= nil) then
		local sortspecs = self.cursorting.specs or {}


		local compare = function(a, b)
			if (a < b) then
				return -1
			end
			if (a > b) then
				return 1
			end
			return 0
		end
		local comparestring = function(a, b)
			--local a = string.lower(as)
			--local b = string.lower(bs)
			if (a < b) then
				return -1
			end
			if (a > b) then
				return 1
			end
			return 0
		end

		local compareVersion = function(a, b)
			if (a == nil and b == nil) then
				return 0
			end
			if (a == nil) then
				a = {}
			end
			if (b == nil) then
				b = {}
			end

			local l = math.max(#a, #b)
			for i = 1, l do
				local ai = a[i] or 0
				local bi = b[i] or 0
				if (ai < bi) then
					return -1
				end
				if (ai > bi) then
					return 1
				end
			end
			return 0
		end

		local compareBool = function(a, b)
			if (not a and b) then
				return -1
			end
			if (a and not b) then
				return 1
			end
			return 0
		end

		local fnbycolumn = {
			compare,
			comparestring,
			comparestring,
			compareVersion,
			compareVersion,
			compareBool,
			compare
		}

		local keybycolumn = {
			"_idx",
			"name_lower",
			"modid_lower",
			"version",
			"upgradeversion",
			"hasSetttings",
			"_updatetime",
			"",
		}
		local sortfn = function(aid, bid)
			local a = mods[aid]
			local b = mods[bid]
			--commonapi.dmp(a)
			--commonapi.dmp(b)
			for k, sortspec in ipairs(sortspecs) do
				local key = keybycolumn[sortspec.columnIndex+1] or ""
				if (key ~= "") then
		
					local fn = fnbycolumn[sortspec.columnIndex+1] or nil
					local delta = fn(a[key], b[key])
					if (delta > 0) then
						return (sortspec.sortDirection == 2)
					elseif (delta < 0) then
						return (sortspec.sortDirection == 1)
					end
				end
			end

			return false
		end
		table.sort(sorted_keys, sortfn)
	end

	for _, idx in ipairs( sorted_keys ) do
		local entry = mods[idx]
		self:genModUIList_CreateRow(uiTableData, entry)
	end
end












return ModListWindow